"""
movie_barcode.

A python movie barcode generator.
"""
from movie_barcode.movies import Movies
from movie_barcode.movie import Movie
from movie_barcode.barcode import BarCode

__version__ = "1.0"
__author__ = 'Marc Bresson'
